import { Subject } from 'rxjs';
import { DragController, DropController, LocalStorage, SessionStorage } from '@anjuna/core';
import { __awaiter, __generator, __extends } from 'tslib';
import { InjectionToken, Inject, Optional, Directive, ElementRef, Input, NgZone, TemplateRef, ViewContainerRef, ContentChildren, Attribute, EventEmitter, HostListener, Output, Renderer2, ChangeDetectorRef, Self, HostBinding, LOCALE_ID, Pipe, Injectable, NgModule, defineInjectable } from '@angular/core';
import { FormGroupDirective, NgForm, NG_VALUE_ACCESSOR, NG_VALIDATORS, NgControl, NG_ASYNC_VALIDATORS } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var LANGUAGE_CODE = new InjectionToken('LanguageCode');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
var BaseTranslationPipe = /** @class */ (function () {
    function BaseTranslationPipe(language) {
        this.language = language;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    BaseTranslationPipe.prototype.transform = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (!this.translations[this.language]) {
            console.warn("Translation file for '" + this.language + "' does not exist.");
            return value;
        }
        return this.translations[this.language][value] || value;
    };
    /** @nocollapse */
    BaseTranslationPipe.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [LANGUAGE_CODE,] }] }
    ]; };
    return BaseTranslationPipe;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@description A wrapper around the HTML 5 Drag API.
 */
var DraggableDirective = /** @class */ (function () {
    function DraggableDirective(element) {
        var _this = this;
        this.element = element;
        this.dataFn = function () { return _this.getData(); };
        this.controller = new DragController(this.element.nativeElement, this.dataFn).connect();
    }
    /**
     * @private
     * @return {?}
     */
    DraggableDirective.prototype.getData = /**
     * @private
     * @return {?}
     */
    function () {
        return this.anjDraggable;
    };
    DraggableDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[anjDraggable]'
                },] }
    ];
    /** @nocollapse */
    DraggableDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    DraggableDirective.propDecorators = {
        anjDraggable: [{ type: Input }]
    };
    return DraggableDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@description A wrapper around the HTML 5 Drop API.
 */
var DroppableDirective = /** @class */ (function () {
    function DroppableDirective(element) {
        this.element = element;
        this.controller = new DropController(this.element.nativeElement).connect();
    }
    DroppableDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[anjDroppable]'
                },] }
    ];
    /** @nocollapse */
    DroppableDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    return DroppableDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DomOutlet = /** @class */ (function () {
    function DomOutlet(targetElement) {
        this.targetElement = targetElement;
    }
    /**
     * @template C
     * @param {?} template
     * @return {?}
     */
    DomOutlet.prototype.attachTemplate = /**
     * @template C
     * @param {?} template
     * @return {?}
     */
    function (template) {
        var _this = this;
        if (this.template.zone.isStable) {
            this._viewRef = this.template.zone.run(function () {
                return template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
            });
        }
        else {
            this._viewRef = template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
            this._viewRef.detectChanges();
        }
        this._viewRef.rootNodes.forEach(function (rootNode) { return _this.targetElement.appendChild(rootNode); });
        this._viewRef.markForCheck();
        this.setDisposeFn((function () {
            /** @type {?} */
            var index = template.viewContainerRef.indexOf(_this._viewRef);
            if (index !== -1) {
                template.viewContainerRef.remove(index);
            }
        }));
        return this._viewRef;
    };
    /**
     * @param {?} context
     * @return {?}
     */
    DomOutlet.prototype.updateContext = /**
     * @param {?} context
     * @return {?}
     */
    function (context) {
        for (var prop in context) {
            if (context.hasOwnProperty(prop)) {
                this._viewRef.context[prop] = context[prop];
            }
        }
        this._viewRef.detectChanges();
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    DomOutlet.prototype.setDisposeFn = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.disposeFn = fn;
    };
    /**
     * @param {?} template
     * @return {?}
     */
    DomOutlet.prototype.attach = /**
     * @param {?} template
     * @return {?}
     */
    function (template) {
        this.template = template;
        return this.attachTemplate(template);
    };
    /**
     * @return {?}
     */
    DomOutlet.prototype.detach = /**
     * @return {?}
     */
    function () {
        if (this.template) {
            this.template.setAttachedHost(null);
            this.template = null;
        }
        this.invokeDisposeFn();
    };
    /**
     * @param {?=} removeParent
     * @return {?}
     */
    DomOutlet.prototype.dispose = /**
     * @param {?=} removeParent
     * @return {?}
     */
    function (removeParent) {
        if (removeParent === void 0) { removeParent = true; }
        this.disposeFn();
        if (removeParent && this.targetElement.parentNode != null) {
            this.targetElement.parentNode.removeChild(this.targetElement);
        }
        return undefined;
    };
    /**
     * @private
     * @return {?}
     */
    DomOutlet.prototype.invokeDisposeFn = /**
     * @private
     * @return {?}
     */
    function () {
        if (this.disposeFn) {
            this.disposeFn();
            this.disposeFn = null;
        }
    };
    return DomOutlet;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var Template = /** @class */ (function () {
    function Template(templateDirective, context) {
        this.templateDirective = templateDirective;
        this.context = context;
    }
    Object.defineProperty(Template.prototype, "origin", {
        get: /**
         * @return {?}
         */
        function () {
            return this.templateDirective.template.elementRef;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Template.prototype, "templateRef", {
        get: /**
         * @return {?}
         */
        function () {
            return this.templateDirective.template;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Template.prototype, "viewContainerRef", {
        get: /**
         * @return {?}
         */
        function () {
            return this.templateDirective.viewContainerRef;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Template.prototype, "zone", {
        get: /**
         * @return {?}
         */
        function () {
            return this.templateDirective.zone;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} host
     * @param {?=} context
     * @return {?}
     */
    Template.prototype.attach = /**
     * @param {?} host
     * @param {?=} context
     * @return {?}
     */
    function (host, context) {
        if (context === void 0) { context = this.context; }
        this.context = context;
        this.host = host;
        return host.attach(this);
    };
    /**
     * @return {?}
     */
    Template.prototype.detach = /**
     * @return {?}
     */
    function () {
        this.context = undefined;
        if (this.host) {
            /** @type {?} */
            var host = this.host;
            this.host = null;
            host.detach();
        }
    };
    /**
     * @param {?} host
     * @return {?}
     */
    Template.prototype.setAttachedHost = /**
     * @param {?} host
     * @return {?}
     */
    function (host) {
        this.host = host;
    };
    return Template;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TemplateDirective = /** @class */ (function () {
    function TemplateDirective(_template, _viewRef, _zone) {
        this._template = _template;
        this._viewRef = _viewRef;
        this._zone = _zone;
    }
    Object.defineProperty(TemplateDirective.prototype, "anjTemplateOf", {
        set: /**
         * @param {?} name
         * @return {?}
         */
        function (name) {
            this._name = name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TemplateDirective.prototype, "name", {
        get: /**
         * @return {?}
         */
        function () {
            return this._name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TemplateDirective.prototype, "template", {
        get: /**
         * @return {?}
         */
        function () {
            return this._template;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TemplateDirective.prototype, "viewContainerRef", {
        get: /**
         * @return {?}
         */
        function () {
            return this._viewRef;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TemplateDirective.prototype, "zone", {
        get: /**
         * @return {?}
         */
        function () {
            return this._zone;
        },
        enumerable: true,
        configurable: true
    });
    TemplateDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[anjTemplate]'
                },] }
    ];
    /** @nocollapse */
    TemplateDirective.ctorParameters = function () { return [
        { type: TemplateRef },
        { type: ViewContainerRef },
        { type: NgZone }
    ]; };
    TemplateDirective.propDecorators = {
        anjTemplateOf: [{ type: Input }]
    };
    return TemplateDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
var AnjTable = /** @class */ (function () {
    function AnjTable(elRef) {
        this.outletRows = [];
        this.element = elRef.nativeElement;
        this.element.renderer = this;
    }
    /**
     * @private
     * @param {?} row
     * @param {?} index
     * @param {?=} column
     * @return {?}
     */
    AnjTable.rowContext = /**
     * @private
     * @param {?} row
     * @param {?} index
     * @param {?=} column
     * @return {?}
     */
    function (row, index, column) {
        return {
            $implicit: row,
            column: column,
            index: index
        };
    };
    /**
     * @private
     * @param {?} outlet
     * @param {?} context
     * @return {?}
     */
    AnjTable.updateContext = /**
     * @private
     * @param {?} outlet
     * @param {?} context
     * @return {?}
     */
    function (outlet, context) {
        outlet.updateContext(context);
    };
    /**
     * @return {?}
     */
    AnjTable.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.outletRows.map(function (rowMap) {
            for (var templateName in rowMap) {
                if (rowMap[templateName]) {
                    rowMap[templateName] = rowMap[templateName].dispose();
                }
            }
            return rowMap;
        });
    };
    /* These next three methods are a mess of duplicated logic - they will probably stay that way until feature complete, then I'll clean em up */
    /* These next three methods are a mess of duplicated logic - they will probably stay that way until feature complete, then I'll clean em up */
    /**
     * @param {?} cell
     * @param {?} offset
     * @param {?} col
     * @param {?} row
     * @param {?} rowIndex
     * @param {?} template
     * @return {?}
     */
    AnjTable.prototype.cellTemplate = /* These next three methods are a mess of duplicated logic - they will probably stay that way until feature complete, then I'll clean em up */
    /**
     * @param {?} cell
     * @param {?} offset
     * @param {?} col
     * @param {?} row
     * @param {?} rowIndex
     * @param {?} template
     * @return {?}
     */
    function (cell, offset, col, row, rowIndex, template) {
        /** @type {?} */
        var rowMap = this.outletRows[rowIndex] || {};
        /** @type {?} */
        var outlet = rowMap[col.name];
        if (!cell) {
            rowMap[col.name] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (cell !== outlet.targetElement) {
                this.updateOutlet(outlet, template, cell, AnjTable.rowContext(row, rowIndex, col));
            }
            else {
                AnjTable.updateContext(outlet, AnjTable.rowContext(row, rowIndex, col));
            }
        }
        else {
            rowMap[col.name] = this.createOutlet(cell, template, AnjTable.rowContext(row, rowIndex, col));
            this.outletRows[rowIndex] = rowMap;
        }
        col.initCell(cell, offset);
    };
    /**
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    AnjTable.prototype.rowTemplate = /**
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    function (templateName, targetEl, row, rowIndex) {
        // Dispose row template no longer used for that index
        this.outletRows[rowIndex] = this.filterRowTemplates(this.outletRows[rowIndex], templateName);
        /** @type {?} */
        var outlet = this.outletRows[rowIndex][templateName];
        if (!targetEl) {
            this.outletRows[rowIndex][templateName] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable.rowContext(row, rowIndex));
            }
            else {
                AnjTable.updateContext(outlet, AnjTable.rowContext(row, rowIndex));
            }
        }
        else {
            this.outletRows[rowIndex][templateName] = this.createOutlet(targetEl, templateName, AnjTable.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = this.outletRows[rowIndex];
        }
    };
    /**
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    AnjTable.prototype.fromTemplate = /**
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    function (templateName, targetEl, row, rowIndex) {
        /** @type {?} */
        var rowMap = this.outletRows[rowIndex] || {};
        /** @type {?} */
        var outlet = rowMap[templateName];
        if (!targetEl) {
            rowMap[templateName] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable.rowContext(row, rowIndex));
            }
            else {
                AnjTable.updateContext(outlet, AnjTable.rowContext(row, rowIndex));
            }
        }
        else {
            rowMap[templateName] = this.createOutlet(targetEl, templateName, AnjTable.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = rowMap;
        }
    };
    /**
     * @private
     * @param {?} target
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    AnjTable.prototype.createOutlet = /**
     * @private
     * @param {?} target
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    function (target, templateName, context) {
        /** @type {?} */
        var outlet = new DomOutlet(target);
        outlet.attach(this.createTemplate(templateName, context));
        return outlet;
    };
    /**
     * @private
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    AnjTable.prototype.createTemplate = /**
     * @private
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    function (templateName, context) {
        /** @type {?} */
        var templateRef = this.templates.find(function (t) { return t.name === templateName; });
        return new Template(templateRef, context);
    };
    /**
     * @private
     * @param {?} outlet
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} context
     * @return {?}
     */
    AnjTable.prototype.updateOutlet = /**
     * @private
     * @param {?} outlet
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} context
     * @return {?}
     */
    function (outlet, templateName, targetEl, context) {
        outlet.targetElement = targetEl;
        outlet.detach();
        outlet.attach(this.createTemplate(templateName, context));
    };
    /**
     * Disposes of row templates no longer used in a given row. This ONLY happens if they have specified more than one
     * Template and the end user sorts or filters or otherwise changes the order. If only one template is specified,
     * the output will always match the input -r
     */
    /**
     * Disposes of row templates no longer used in a given row. This ONLY happens if they have specified more than one
     * Template and the end user sorts or filters or otherwise changes the order. If only one template is specified,
     * the output will always match the input -r
     * @private
     * @param {?=} rowMap
     * @param {?=} templateName
     * @return {?}
     */
    AnjTable.prototype.filterRowTemplates = /**
     * Disposes of row templates no longer used in a given row. This ONLY happens if they have specified more than one
     * Template and the end user sorts or filters or otherwise changes the order. If only one template is specified,
     * the output will always match the input -r
     * @private
     * @param {?=} rowMap
     * @param {?=} templateName
     * @return {?}
     */
    function (rowMap, templateName) {
        if (rowMap === void 0) { rowMap = {}; }
        /** @type {?} */
        var updated = {};
        Object.keys(rowMap).forEach(function (k) {
            if (k === templateName) {
                updated[k] = rowMap[k];
            }
            else {
                rowMap[k].dispose(false);
            }
        });
        return updated;
    };
    AnjTable.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-table:not([anjIgnore])',
                },] }
    ];
    /** @nocollapse */
    AnjTable.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    AnjTable.propDecorators = {
        templates: [{ type: ContentChildren, args: [TemplateDirective,] }]
    };
    return AnjTable;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
var TransitionState = {
    Enter: 0,
    Leave: 1,
    EnterTo: 2,
    Inactive: 3,
};
TransitionState[TransitionState.Enter] = 'Enter';
TransitionState[TransitionState.Leave] = 'Leave';
TransitionState[TransitionState.EnterTo] = 'EnterTo';
TransitionState[TransitionState.Inactive] = 'Inactive';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@experimental
 */
var TransitionDirective = /** @class */ (function () {
    function TransitionDirective(renderer, element, mode, name) {
        this.renderer = renderer;
        this.element = element;
        this.mode = mode;
        this.name = name;
        this.state = TransitionState.EnterTo;
        this.end = new EventEmitter();
        this.mode = this.mode ? this.mode : 'in-out';
        this.name = this.name ? this.name : 'transition';
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    TransitionDirective.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        this.endTransition();
        if (!changes.anjTransition.previousValue && !changes.anjTransition.currentValue && this.mode !== 'out') {
            this.initEnter();
        }
        else if (!changes.anjTransition.previousValue && changes.anjTransition.currentValue) {
            this.transitionIn();
        }
        else {
            this.transitionOut();
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    TransitionDirective.prototype.handleTransition = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (event.target === this.element.nativeElement && TransitionDirective.supportedTransitions.indexOf(event.propertyName) > -1) {
            event.stopPropagation();
            this.end.emit(this.state);
            if (this.state === TransitionState.Enter) {
                this.endTransition();
            }
            else {
                if (this.mode !== 'out' && this.anjTransition) {
                    this.transitionIn();
                }
                else {
                    this.endTransition();
                }
            }
        }
    };
    /**
     * @private
     * @return {?}
     */
    TransitionDirective.prototype.initEnter = /**
     * @private
     * @return {?}
     */
    function () {
        this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
    };
    /**
     * @private
     * @return {?}
     */
    TransitionDirective.prototype.transitionIn = /**
     * @private
     * @return {?}
     */
    function () {
        if (this.mode !== 'out') {
            this.state = TransitionState.Enter;
            this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
            this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-leave");
            this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-leave-active");
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter-active");
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter");
        }
        else {
            this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
            this.endTransition();
        }
    };
    /**
     * @private
     * @return {?}
     */
    TransitionDirective.prototype.transitionOut = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        this.state = TransitionState.Leave;
        if (this.mode !== 'in') {
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-leave-active");
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-leave");
        }
        else {
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
            this.state = TransitionState.EnterTo;
            if (this.anjTransition) {
                window.setTimeout(function () {
                    _this.transitionIn();
                });
            }
        }
    };
    /**
     * @private
     * @return {?}
     */
    TransitionDirective.prototype.endTransition = /**
     * @private
     * @return {?}
     */
    function () {
        this.state = TransitionState.Inactive;
        this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-enter-active");
        this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-leave-active");
        this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-enter");
        this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-leave");
        if (!this.anjTransition) {
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
        }
    };
    TransitionDirective.supportedTransitions = ['opacity', 'transform'];
    TransitionDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[anjTransition]'
                },] }
    ];
    /** @nocollapse */
    TransitionDirective.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef },
        { type: undefined, decorators: [{ type: Attribute, args: ['mode',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['name',] }] }
    ]; };
    TransitionDirective.propDecorators = {
        anjTransition: [{ type: Input }],
        end: [{ type: Output }],
        handleTransition: [{ type: HostListener, args: ['transitiend', ['$event'],] }]
    };
    return TransitionDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ValueAccessor = /** @class */ (function () {
    function ValueAccessor(element, parentForm, parentGroup) {
        this.element = element;
        /**
         * Enable two-way binding
         */
        this.valueChange = new EventEmitter();
        this.onChange = function () { };
        this.onTouched = function () { };
        this.parent = parentForm ? parentForm : parentGroup ? parentGroup : null;
        if (this.parent) {
            this.formSub = this.parent.ngSubmit.subscribe(this.checkValidity.bind(this));
        }
    }
    /**
     * @return {?}
     */
    ValueAccessor.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        if (this.formSub) {
            this.formSub.unsubscribe();
        }
    };
    /**
     * @param {?} value
     * @return {?}
     */
    ValueAccessor.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.element.nativeElement.value = this.lastValue = value == null ? '' : value;
    };
    /**
     * @param {?} value
     * @return {?}
     */
    ValueAccessor.prototype.onChangeEvent = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value !== this.lastValue) {
            this.lastValue = value;
            this.onChange(value);
            this.valueChange.emit(value);
        }
    };
    /**
     * @return {?}
     */
    ValueAccessor.prototype.onAnjBlur = /**
     * @return {?}
     */
    function () {
        this.onTouched();
        this.checkValidity();
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    ValueAccessor.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    ValueAccessor.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onTouched = fn;
    };
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    ValueAccessor.prototype.setDisabledState = /**
     * @param {?} isDisabled
     * @return {?}
     */
    function (isDisabled) {
        this.element.nativeElement.disabled = isDisabled;
    };
    /**
     * @return {?}
     */
    ValueAccessor.prototype.checkValidity = /**
     * @return {?}
     */
    function () {
        // TODO: this won't work for custom validators
        this.element.nativeElement.checkValidity();
    };
    ValueAccessor.propDecorators = {
        valueChange: [{ type: Output }],
        onAnjBlur: [{ type: HostListener, args: ['anjBlur',] }]
    };
    return ValueAccessor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
var BooleanValueDirective = /** @class */ (function (_super) {
    __extends(BooleanValueDirective, _super);
    function BooleanValueDirective(parentForm, parentFormGroup, element) {
        return _super.call(this, element, parentForm, parentFormGroup) || this;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    BooleanValueDirective.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.element.nativeElement.checked = this.lastValue = value == null ? false : value;
    };
    /**
     * @param {?} value
     * @return {?}
     */
    BooleanValueDirective.prototype.onAnjChange = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.onChangeEvent(value);
        this.checkValidity();
    };
    BooleanValueDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-checkbox, anj-toggle',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: BooleanValueDirective,
                            multi: true
                        }
                    ]
                },] }
    ];
    /** @nocollapse */
    BooleanValueDirective.ctorParameters = function () { return [
        { type: NgForm, decorators: [{ type: Optional }] },
        { type: FormGroupDirective, decorators: [{ type: Optional }] },
        { type: ElementRef }
    ]; };
    BooleanValueDirective.propDecorators = {
        onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.target.checked'],] }]
    };
    return BooleanValueDirective;
}(ValueAccessor));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@dynamic
 * @abstract
 */
var AnjunaValidators = /** @class */ (function () {
    function AnjunaValidators() {
    }
    /**
     * @param {?} c
     * @return {?}
     */
    AnjunaValidators.date = /**
     * @param {?} c
     * @return {?}
     */
    function (c) {
        if (!c.value) {
            return;
        }
        /** @type {?} */
        var isValid = true;
        // Handle obvious Invalid Date
        if (c.value.toString() === 'Invalid Date') {
            isValid = false;
            // Handle Moment Date
        }
        else if (typeof c.value._isValid === 'boolean') {
            isValid = c.value._isValid;
            // Handle Native Date
        }
        else {
            isValid = !isNaN(new Date(c.value).getTime());
        }
        if (isValid || c.value === '') {
            return null;
        }
        else {
            return {
                date: {
                    valid: false
                }
            };
        }
    };
    /**
     * @param {?} c
     * @return {?}
     */
    AnjunaValidators.requiredCheck = /**
     * @param {?} c
     * @return {?}
     */
    function (c) {
        if (c.value) {
            return null;
        }
        else {
            return {
                required: {
                    valid: false
                }
            };
        }
    };
    /**
     * @param {?} c
     * @return {?}
     */
    AnjunaValidators.unique = /**
     * @param {?} c
     * @return {?}
     */
    function (c) {
        if (!c.value || !Array.isArray(c.value)) {
            return;
        }
        /** @type {?} */
        var testArray = c.value.filter(Boolean);
        /** @type {?} */
        var isValid = testArray.length === Array.from(new Set(testArray)).length;
        if (isValid) {
            return null;
        }
        else {
            return {
                unique: {
                    valid: false
                }
            };
        }
    };
    AnjunaValidators.minDate = function (min) {
        return function (c) {
            if (!c.value) {
                return;
            }
            if (c.value < new Date(min)) {
                return {
                    min: {
                        valid: false
                    }
                };
            }
            else {
                return null;
            }
        };
    };
    AnjunaValidators.maxDate = function (max) {
        return function (c) {
            if (!c.value) {
                return;
            }
            if (c.value > new Date(max)) {
                return {
                    max: {
                        valid: false
                    }
                };
            }
            else {
                return null;
            }
        };
    };
    return AnjunaValidators;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
var DateValidatorDirective = /** @class */ (function () {
    function DateValidatorDirective() {
    }
    /**
     * @param {?} c
     * @return {?}
     */
    DateValidatorDirective.prototype.validate = /**
     * @param {?} c
     * @return {?}
     */
    function (c) {
        return AnjunaValidators.date(c);
    };
    DateValidatorDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[date][ngModel],[date][formControl]',
                    providers: [
                        { provide: NG_VALIDATORS, useExisting: DateValidatorDirective, multi: true }
                    ]
                },] }
    ];
    return DateValidatorDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var TIME_TIMEZONE_STRING = new InjectionToken('TimeTimezoneString');
/** @type {?} */
var TIME_MERIDIEM = new InjectionToken('TimeMeridiem');
/** @type {?} */
var DATE_TIMEZONE_OVERRIDE = new InjectionToken('DateTimezoneOverride');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DatePickerDirective = /** @class */ (function () {
    function DatePickerDirective(renderer, _element, changeRef, _locale, _timezone, ngControl) {
        this.renderer = renderer;
        this._element = _element;
        this.changeRef = changeRef;
        this._locale = _locale;
        this._timezone = _timezone;
        this.ngControl = ngControl;
        this.onChange = function () { };
        this.onTouch = function () { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    Object.defineProperty(DatePickerDirective.prototype, "value", {
        // Custom input to return inner value, but display masked value
        get: 
        // Custom input to return inner value, but display masked value
        /**
         * @return {?}
         */
        function () { return this.innerDateValue; },
        set: /**
         * @param {?} dateValue
         * @return {?}
         */
        function (dateValue) {
            if (dateValue) {
                this.outerValue = this.prepDateStringOutput(dateValue);
                this.changeRef.markForCheck();
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    DatePickerDirective.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        this.picker = this._element.nativeElement.querySelector('anj-date-picker');
        if ((this.ngControl && this.ngControl.value) || this.ngModel) {
            this.value = this.ngModel || this.ngControl.value;
            this.changeRef.detectChanges();
        }
    };
    /**
     * @return {?}
     */
    DatePickerDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        if (this.listenerFn) {
            this.listenerFn();
        }
    };
    // // Start: ControlValueAccessor requirements
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    DatePickerDirective.prototype.writeValue = 
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    function (newValue) {
        this.value = newValue;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    DatePickerDirective.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    DatePickerDirective.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onTouch = fn;
    };
    // // End: ControlValueAccessor requirements
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    DatePickerDirective.prototype.onInput = 
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (event.target.tagName === 'ANJ-INPUT') {
            this.onTouch();
            this.updateInnerValue(event.target.value);
            this.valueChange.emit(this.innerDateValue);
            this._element.nativeElement.checkValidity();
            this.changeRef.markForCheck();
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    DatePickerDirective.prototype.onCalendarSelect = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.onTouch();
        this.updateInnerValue(event.detail);
        this.valueChange.emit(this.innerDateValue);
        this._element.nativeElement.checkValidity();
        this.changeRef.markForCheck();
    };
    // convert the masked value to storage value
    // convert the masked value to storage value
    /**
     * @private
     * @param {?} inputDate
     * @return {?}
     */
    DatePickerDirective.prototype.updateInnerValue = 
    // convert the masked value to storage value
    /**
     * @private
     * @param {?} inputDate
     * @return {?}
     */
    function (inputDate) {
        this.innerDateValue = this.picker.adapter.convertToDate(inputDate);
        this.propogateInnerValueChange();
    };
    /**
     * @private
     * @return {?}
     */
    DatePickerDirective.prototype.propogateInnerValueChange = /**
     * @private
     * @return {?}
     */
    function () {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
        this.valueChange.emit(this.innerDateValue);
    };
    /**
     * @private
     * @param {?} input
     * @return {?}
     */
    DatePickerDirective.prototype.prepDateStringOutput = /**
     * @private
     * @param {?} input
     * @return {?}
     */
    function (input) {
        if (!input) {
            return null;
        }
        else if (typeof input === 'string') {
            return input;
        }
        else if (this._timezone) {
            // Timezone-override aware formatting
            /** @type {?} */
            var tzTime = new Date(input).toLocaleString(undefined, { timeZone: this._timezone });
            return new Date(tzTime).toLocaleDateString(this._locale);
        }
        else {
            return new Date(input).toLocaleDateString(this._locale, { month: '2-digit', day: '2-digit', year: 'numeric' });
        }
    };
    DatePickerDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-input[anjDatePicker]',
                    exportAs: 'anjDatepickerInput'
                },] }
    ];
    /** @nocollapse */
    DatePickerDirective.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [DATE_TIMEZONE_OVERRIDE,] }] },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
    ]; };
    DatePickerDirective.propDecorators = {
        value: [{ type: Input }],
        ngModel: [{ type: Input }],
        outerValue: [{ type: HostBinding, args: ['value',] }],
        onInput: [{ type: HostListener, args: ['anjBlur', ['$event'],] }],
        onCalendarSelect: [{ type: HostListener, args: ['anjCalendarSelection', ['$event'],] }]
    };
    return DatePickerDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
var TextValueDirective = /** @class */ (function (_super) {
    __extends(TextValueDirective, _super);
    function TextValueDirective(parentForm, parentFormGroup, element) {
        return _super.call(this, element, parentForm, parentFormGroup) || this;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    TextValueDirective.prototype.onAnjChange = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.onChangeEvent(value);
        if (this.element.nativeElement.invalid) {
            this.checkValidity();
        }
    };
    TextValueDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-input, anj-textarea',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: TextValueDirective,
                            multi: true
                        }
                    ]
                },] }
    ];
    /** @nocollapse */
    TextValueDirective.ctorParameters = function () { return [
        { type: NgForm, decorators: [{ type: Optional }] },
        { type: FormGroupDirective, decorators: [{ type: Optional }] },
        { type: ElementRef }
    ]; };
    TextValueDirective.propDecorators = {
        onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.detail.value'],] }]
    };
    return TextValueDirective;
}(ValueAccessor));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
var RequiredCheckDirective = /** @class */ (function () {
    function RequiredCheckDirective() {
    }
    /**
     * @param {?} c
     * @return {?}
     */
    RequiredCheckDirective.prototype.validate = /**
     * @param {?} c
     * @return {?}
     */
    function (c) {
        return AnjunaValidators.requiredCheck(c);
    };
    RequiredCheckDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-checkbox[required][ngModel], anj-toggle[required][ngModel]',
                    providers: [
                        { provide: NG_VALIDATORS, useExisting: RequiredCheckDirective, multi: true }
                    ]
                },] }
    ];
    return RequiredCheckDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
var SelectValueDirective = /** @class */ (function (_super) {
    __extends(SelectValueDirective, _super);
    function SelectValueDirective(parentForm, parentFormGroup, element) {
        return _super.call(this, element, parentForm, parentFormGroup) || this;
    }
    /**
     * @param {?} target
     * @return {?}
     */
    SelectValueDirective.prototype.onAnjChange = /**
     * @param {?} target
     * @return {?}
     */
    function (target) {
        // ignore the input for type-aheads
        if (target.matches('anj-input')) {
            return;
        }
        this.onChangeEvent(target.value);
        this.checkValidity();
    };
    SelectValueDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-radio-group, anj-select, anj-type-ahead',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: SelectValueDirective,
                            multi: true
                        }
                    ]
                },] }
    ];
    /** @nocollapse */
    SelectValueDirective.ctorParameters = function () { return [
        { type: NgForm, decorators: [{ type: Optional }] },
        { type: FormGroupDirective, decorators: [{ type: Optional }] },
        { type: ElementRef }
    ]; };
    SelectValueDirective.propDecorators = {
        onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.target'],] }]
    };
    return SelectValueDirective;
}(ValueAccessor));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TimePickerDirective = /** @class */ (function () {
    function TimePickerDirective(_element, ngControl, _timezone, _meridiem) {
        this._element = _element;
        this.ngControl = ngControl;
        this._timezone = _timezone;
        this._meridiem = _meridiem;
        this.onChange = function () { };
        this.onTouch = function () { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    TimePickerDirective.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var picker = this._element.nativeElement.querySelector('anj-time-picker');
        picker.setAttribute('meridiem', this._meridiem);
        if (this._timezone) {
            picker.setAttribute('timezone', this._timezone);
        }
    };
    // // Start: ControlValueAccessor requirements
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    TimePickerDirective.prototype.writeValue = 
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    function (newValue) {
        this.value = newValue;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    TimePickerDirective.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    TimePickerDirective.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onTouch = fn;
    };
    // // End: ControlValueAccessor requirements
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    TimePickerDirective.prototype.onInput = 
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.value = event.detail;
        if (this.ngControl) {
            this.ngControl.control.setValue(this.value);
        }
        this.valueChange.emit(this.value);
    };
    TimePickerDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-input[anjTimePicker]',
                    exportAs: 'anjTimepickerInput'
                },] }
    ];
    /** @nocollapse */
    TimePickerDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [TIME_TIMEZONE_STRING,] }] },
        { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
    ]; };
    TimePickerDirective.propDecorators = {
        value: [{ type: HostBinding, args: ['value',] }],
        onInput: [{ type: HostListener, args: ['anjClockSelection', ['$event'],] }]
    };
    return TimePickerDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ZuoraDatePipe = /** @class */ (function () {
    function ZuoraDatePipe(_locale) {
        this._locale = _locale;
    }
    /**
     * @param {?} value
     * @param {?=} args
     * @return {?}
     */
    ZuoraDatePipe.prototype.transform = /**
     * @param {?} value
     * @param {?=} args
     * @return {?}
     */
    function (value, args) {
        try {
            /** @type {?} */
            var valueAsDate = new Date(value);
            /** @type {?} */
            var userOpts = (/** @type {?} */ (args));
            /** @type {?} */
            var defaultOpts = { year: '2-digit', month: 'numeric', day: 'numeric' };
            /** @type {?} */
            var opts = Object.assign({}, defaultOpts, userOpts);
            return new Intl.DateTimeFormat(this._locale, opts).format(valueAsDate);
        }
        catch (e) {
            return value.toString();
        }
    };
    ZuoraDatePipe.decorators = [
        { type: Pipe, args: [{
                    name: 'zuoraDate'
                },] }
    ];
    /** @nocollapse */
    ZuoraDatePipe.ctorParameters = function () { return [
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] }
    ]; };
    return ZuoraDatePipe;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ToastService = /** @class */ (function () {
    function ToastService() {
        this._dismissed$ = new Subject();
    }
    Object.defineProperty(ToastService.prototype, "dismissed$", {
        get: /**
         * @return {?}
         */
        function () {
            return this._dismissed$.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ToastService.createController = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var controller = document.createElement('anj-toast-controller');
        document.body.appendChild(controller);
        return (/** @type {?} */ (controller));
    };
    /**
     * @return {?}
     */
    ToastService.prototype.dismissAll = /**
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.dismissAll()];
                }
            });
        });
    };
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    ToastService.prototype.error = /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    function (message, header) {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.error(message, header)];
                }
            });
        });
    };
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    ToastService.prototype.message = /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    function (message, header) {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.message(message, header)];
                }
            });
        });
    };
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    ToastService.prototype.success = /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    function (message, header) {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.success(message, header)];
                }
            });
        });
    };
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    ToastService.prototype.warn = /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    function (message, header) {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.warn(message, header)];
                }
            });
        });
    };
    /**
     * @return {?}
     */
    ToastService.prototype.getController = /**
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.controller || this.findController()];
            });
        });
    };
    /**
     * @private
     * @return {?}
     */
    ToastService.prototype.findController = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var controller = this.findOrCreate(document.querySelector('anj-toast-controller'));
        controller.addEventListener('anjToastDidDismiss', this.onToastDidDismiss.bind(this));
        this.controller = controller;
        return controller.componentOnReady();
    };
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    ToastService.prototype.onToastDidDismiss = /**
     * @private
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this._dismissed$.next(event);
    };
    /**
     * @private
     * @param {?} control
     * @return {?}
     */
    ToastService.prototype.findOrCreate = /**
     * @private
     * @param {?} control
     * @return {?}
     */
    function (control) {
        return control || ToastService.createController();
    };
    ToastService.decorators = [
        { type: Injectable, args: [{ providedIn: 'root' },] }
    ];
    /** @nocollapse */ ToastService.ngInjectableDef = defineInjectable({ factory: function ToastService_Factory() { return new ToastService(); }, token: ToastService, providedIn: "root" });
    return ToastService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var LocalStorageService = /** @class */ (function (_super) {
    __extends(LocalStorageService, _super);
    function LocalStorageService() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LocalStorageService.decorators = [
        { type: Injectable, args: [{ providedIn: 'root' },] }
    ];
    /** @nocollapse */ LocalStorageService.ngInjectableDef = defineInjectable({ factory: function LocalStorageService_Factory() { return new LocalStorageService(); }, token: LocalStorageService, providedIn: "root" });
    return LocalStorageService;
}(LocalStorage));
var SessionStorageService = /** @class */ (function (_super) {
    __extends(SessionStorageService, _super);
    function SessionStorageService() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SessionStorageService.decorators = [
        { type: Injectable, args: [{ providedIn: 'root' },] }
    ];
    /** @nocollapse */ SessionStorageService.ngInjectableDef = defineInjectable({ factory: function SessionStorageService_Factory() { return new SessionStorageService(); }, token: SessionStorageService, providedIn: "root" });
    return SessionStorageService;
}(SessionStorage));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
var LocaleDirective = /** @class */ (function () {
    function LocaleDirective(changeRef, _locale, _lang, _timezone) {
        this.changeRef = changeRef;
        if (_locale) {
            this._locale = _locale;
            this._lang = _lang ? _lang : this._locale ? this._locale.split('-')[0] : 'en';
        }
        if (_timezone) {
            this._timezone = _timezone;
        }
        this.changeRef.markForCheck();
    }
    LocaleDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-locale',
                },] }
    ];
    /** @nocollapse */
    LocaleDirective.ctorParameters = function () { return [
        { type: ChangeDetectorRef },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LANGUAGE_CODE,] }] },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [DATE_TIMEZONE_OVERRIDE,] }] }
    ]; };
    LocaleDirective.propDecorators = {
        _locale: [{ type: HostBinding, args: ['attr.locale',] }],
        _lang: [{ type: HostBinding, args: ['attr.lang',] }],
        _timezone: [{ type: HostBinding, args: ['attr.timezone',] }]
    };
    return LocaleDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
var CustomNativeErrorsDirective = /** @class */ (function () {
    function CustomNativeErrorsDirective(el, changeRef) {
        this.el = el;
        this.changeRef = changeRef;
    }
    /**
     * @param {?} c
     * @return {?}
     */
    CustomNativeErrorsDirective.prototype.validate = /**
     * @param {?} c
     * @return {?}
     */
    function (c) {
        return __awaiter(this, void 0, void 0, function () {
            var validityErrors;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.el.nativeElement.checkCustomValidity) {
                            return [2 /*return*/, null];
                        }
                        return [4 /*yield*/, this.el.nativeElement.checkCustomValidity()];
                    case 1:
                        validityErrors = _a.sent();
                        if (!validityErrors) {
                            return [2 /*return*/, null];
                        }
                        else {
                            this.changeRef.markForCheck();
                            return [2 /*return*/, validityErrors];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    CustomNativeErrorsDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-input[anjDatePicker], anj-input[anjTimePicker], anj-input[anjDatetimePicker]',
                    providers: [
                        { provide: NG_ASYNC_VALIDATORS, useExisting: CustomNativeErrorsDirective, multi: true }
                    ]
                },] }
    ];
    /** @nocollapse */
    CustomNativeErrorsDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ChangeDetectorRef }
    ]; };
    return CustomNativeErrorsDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DatetimePickerDirective = /** @class */ (function () {
    function DatetimePickerDirective(_element, changeRef, ngControl, _locale, _meridiem) {
        this._element = _element;
        this.changeRef = changeRef;
        this.ngControl = ngControl;
        this._locale = _locale;
        this._meridiem = _meridiem;
        this.onChange = function () { };
        this.onTouch = function () { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    DatetimePickerDirective.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        this.picker = this._element.nativeElement.querySelector('anj-datetime-picker');
        this.picker.setAttribute('meridiem', this._meridiem);
        if ((this.ngControl && this.ngControl.value) || this.ngModel) {
            this.innerDateValue = this.ngModel || this.ngControl.value;
            /** @type {?} */
            var d = new Date(this.innerDateValue);
            this.outerValue = this.localizeForOuterValue(d);
            this.changeRef.detectChanges();
        }
    };
    // Start: ControlValueAccessor requirements
    // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    DatetimePickerDirective.prototype.writeValue = 
    // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    function (newValue) {
        this.innerDateValue = newValue;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    DatetimePickerDirective.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    DatetimePickerDirective.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onTouch = fn;
    };
    // End: ControlValueAccessor requirements
    // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    DatetimePickerDirective.prototype.onInput = 
    // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (event.target.tagName === 'ANJ-INPUT') {
            this.picker.processInputValue(event.target.value);
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    DatetimePickerDirective.prototype.onAnjDatetimeChange = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.changeHandler(event.detail);
    };
    /**
     * @private
     * @param {?} changedVal
     * @return {?}
     */
    DatetimePickerDirective.prototype.changeHandler = /**
     * @private
     * @param {?} changedVal
     * @return {?}
     */
    function (changedVal) {
        this.onTouch();
        this.outerValue = changedVal.outer;
        this.innerDateValue = changedVal.inner;
        this.propogateInnerValueChange();
        this.changeRef.markForCheck();
    };
    /**
     * @private
     * @param {?} date
     * @return {?}
     */
    DatetimePickerDirective.prototype.localizeForOuterValue = /**
     * @private
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return date.toLocaleString(this._locale, { hour12: !!this._meridiem, month: '2-digit', day: '2-digit', year: 'numeric', hour: '2-digit', minute: '2-digit' });
    };
    /**
     * @private
     * @return {?}
     */
    DatetimePickerDirective.prototype.propogateInnerValueChange = /**
     * @private
     * @return {?}
     */
    function () {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
    };
    DatetimePickerDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'anj-input[anjDatetimePicker]',
                    exportAs: 'anjDatetimePickerInput'
                },] }
    ];
    /** @nocollapse */
    DatetimePickerDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
        { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
    ]; };
    DatetimePickerDirective.propDecorators = {
        ngModel: [{ type: Input }],
        outerValue: [{ type: HostBinding, args: ['value',] }],
        onInput: [{ type: HostListener, args: ['anjBlur', ['$event'],] }],
        onAnjDatetimeChange: [{ type: HostListener, args: ['anjDatetimeChange', ['$event'],] }]
    };
    return DatetimePickerDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var DECLARATIONS = [
    // Common
    DraggableDirective,
    DroppableDirective,
    LocaleDirective,
    TemplateDirective,
    TransitionDirective,
    ZuoraDatePipe,
    // Forms
    BooleanValueDirective,
    CustomNativeErrorsDirective,
    DatePickerDirective,
    DatetimePickerDirective,
    DateValidatorDirective,
    RequiredCheckDirective,
    SelectValueDirective,
    TextValueDirective,
    TimePickerDirective,
    // Tables
    AnjTable
];
var AnjunaCoreModule = /** @class */ (function () {
    function AnjunaCoreModule() {
    }
    AnjunaCoreModule.decorators = [
        { type: NgModule, args: [{
                    declarations: DECLARATIONS,
                    exports: DECLARATIONS,
                    providers: [
                        {
                            provide: TIME_MERIDIEM,
                            useValue: true
                        },
                        ZuoraDatePipe
                    ]
                },] }
    ];
    return AnjunaCoreModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { BaseTranslationPipe, DraggableDirective, DroppableDirective, AnjTable, TemplateDirective, TransitionDirective, BooleanValueDirective, DateValidatorDirective, DatePickerDirective, TextValueDirective, RequiredCheckDirective, SelectValueDirective, TimePickerDirective, LANGUAGE_CODE, TIME_TIMEZONE_STRING, TIME_MERIDIEM, DATE_TIMEZONE_OVERRIDE, DomOutlet, Template, AnjunaValidators, ZuoraDatePipe, ToastService, LocalStorageService, SessionStorageService, TransitionState, AnjunaCoreModule, LocaleDirective as ɵb, CustomNativeErrorsDirective as ɵc, DatetimePickerDirective as ɵd, ValueAccessor as ɵa };

//# sourceMappingURL=anjuna-angular-core.js.map